print("import: 'cachy'")
import cachy

print("import: 'cachy.contracts'")
import cachy.contracts

print("import: 'cachy.serializers'")
import cachy.serializers

print("import: 'cachy.stores'")
import cachy.stores

